@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -50,6 +42,7 @@
 #include <private/qanimationgroupjob_p.h>
 
 #include <qcoreapplication.h>
+#include <qdebug.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -57,6 +50,7 @@ QQuickAnimatorProxyJob::QQuickAnimatorProxyJob(QAbstractAnimationJob *job, QObje
     : m_controller(0)
     , m_job(job)
     , m_internalState(State_Stopped)
+    , m_jobManagedByController(false)
 {
     m_isRenderThreadProxy = true;
     m_animation = qobject_cast<QQuickAbstractAnimation *>(item);
@@ -92,14 +86,21 @@ QQuickAnimatorProxyJob::QQuickAnimatorProxyJob(QAbstractAnimationJob *job, QObje
 QQuickAnimatorProxyJob::~QQuickAnimatorProxyJob()
 {
     deleteJob();
+    if (m_controller)
+        m_controller->proxyWasDestroyed(this);
 }
 
 void QQuickAnimatorProxyJob::deleteJob()
 {
     if (m_job) {
-        if (m_controller && m_internalState != State_Starting)
+        // If we have a controller, we might have posted the job to be started
+        // so delete it through the controller to clean up properly.
+        if (m_controller)
             m_controller->deleteJob(m_job);
-        else if (m_internalState == State_Starting)
+
+        // We explicitly delete the job if the animator controller has never touched
+        // it. If it has, it will have ownership as well.
+        else if (!m_jobManagedByController)
             delete m_job;
         m_job = 0;
     }
@@ -133,6 +134,13 @@ void QQuickAnimatorProxyJob::updateState(QAbstractAnimationJob::State newState,
     }
 }
 
+void QQuickAnimatorProxyJob::debugAnimation(QDebug d) const
+{
+    d << "QuickAnimatorProxyJob("<< hex << (const void *) this << dec
+      << "state:" << state() << "duration:" << duration()
+      << "proxying: (" << job() << ')';
+}
+
 void QQuickAnimatorProxyJob::windowChanged(QQuickWindow *window)
 {
     setWindow(window);
@@ -147,11 +155,13 @@ void QQuickAnimatorProxyJob::controllerWasDeleted()
 void QQuickAnimatorProxyJob::setWindow(QQuickWindow *window)
 {
     if (!window) {
-        // Stop will trigger syncBackCurrentValues so best to do it before
-        // we delete m_job.
         stop();
         deleteJob();
-        return;
+
+        // Upon leaving a window, we reset the controller. This means that
+        // animators will only enter the Starting phase and won't be making
+        // calls to QQuickAnimatorController::startjob().
+        m_controller = 0;
 
     } else if (!m_controller && m_job) {
         m_controller = QQuickWindowPrivate::get(window)->animationController;
@@ -179,15 +189,6 @@ void QQuickAnimatorProxyJob::startedByController()
     m_internalState = State_Running;
 }
 
-bool QQuickAnimatorProxyJob::event(QEvent *e)
-{
-    if (e->type() == QEvent::User) {
-        stop();
-        return true;
-    }
-    return QObject::event(e);
-}
-
 static void qquick_syncback_helper(QAbstractAnimationJob *job)
 {
     if (job->isRenderThreadJob()) {
@@ -222,6 +223,13 @@ QQuickAnimatorJob::QQuickAnimatorJob()
     m_isRenderThreadJob = true;
 }
 
+void QQuickAnimatorJob::debugAnimation(QDebug d) const
+{
+    d << "QuickAnimatorJob(" << hex << (const void *) this << dec
+      << ") state:" << state() << "duration:" << duration()
+      << "target:" << m_target << "value:" << m_value;
+}
+
 qreal QQuickAnimatorJob::value() const
 {
     qreal v;
@@ -256,6 +264,10 @@ QQuickTransformAnimatorJob::QQuickTransformAnimatorJob()
 QQuickTransformAnimatorJob::~QQuickTransformAnimatorJob()
 {
     if (m_helper && --m_helper->ref == 0) {
+        // The only condition for not having a controller is when target was
+        // destroyed, in which case we have neither m_helper nor m_contorller.
+        Q_ASSERT(m_controller);
+        Q_ASSERT(m_helper->item);
         m_controller->m_transforms.remove(m_helper->item);
         delete m_helper;
     }
@@ -266,6 +278,7 @@ void QQuickTransformAnimatorJob::initialize(QQuickAnimatorController *controller
     QQuickAnimatorJob::initialize(controller);
 
     if (m_controller) {
+        bool newHelper = m_helper == 0;
         m_helper = m_controller->m_transforms.value(m_target);
         if (!m_helper) {
             m_helper = new Helper();
@@ -273,7 +286,8 @@ void QQuickTransformAnimatorJob::initialize(QQuickAnimatorController *controller
             m_controller->m_transforms.insert(m_target, m_helper);
             QObject::connect(m_target, SIGNAL(destroyed(QObject*)), m_controller, SLOT(itemDestroyed(QObject*)), Qt::DirectConnection);
         } else {
-            ++m_helper->ref;
+            if (newHelper) // only add reference the first time around..
+                ++m_helper->ref;
             // Make sure leftovers from previous runs are being used...
             m_helper->wasSynced = false;
         }
@@ -281,6 +295,18 @@ void QQuickTransformAnimatorJob::initialize(QQuickAnimatorController *controller
     }
 }
 
+void QQuickTransformAnimatorJob::nodeWasDestroyed()
+{
+    if (m_helper)
+        m_helper->node = 0;
+}
+
+void QQuickTransformAnimatorJob::targetWasDeleted()
+{
+    m_helper = 0;
+    QQuickAnimatorJob::targetWasDeleted();
+}
+
 void QQuickTransformAnimatorJob::Helper::sync()
 {
     const quint32 mask = QQuickItemPrivate::Position
@@ -326,7 +352,7 @@ void QQuickTransformAnimatorJob::Helper::sync()
 
 void QQuickTransformAnimatorJob::Helper::apply()
 {
-    if (!wasChanged)
+    if (!wasChanged || !node)
         return;
 
     QMatrix4x4 m;
@@ -352,7 +378,7 @@ void QQuickXAnimatorJob::updateCurrentTime(int time)
 {
     if (!m_controller)
         return;
-    Q_ASSERT(m_controller->m_window->openglContext()->thread() == QThread::currentThread());
+    Q_ASSERT(!m_controller->m_window->openglContext() || m_controller->m_window->openglContext()->thread() == QThread::currentThread());
 
     m_value = m_from + (m_to - m_from) * m_easing.valueForProgress(time / (qreal) m_duration);
     m_helper->dx = m_value;
@@ -369,7 +395,7 @@ void QQuickYAnimatorJob::updateCurrentTime(int time)
 {
     if (!m_controller)
         return;
-    Q_ASSERT(m_controller->m_window->openglContext()->thread() == QThread::currentThread());
+    Q_ASSERT(!m_controller->m_window->openglContext() || m_controller->m_window->openglContext()->thread() == QThread::currentThread());
 
     m_value = m_from + (m_to - m_from) * m_easing.valueForProgress(time / (qreal) m_duration);
     m_helper->dy = m_value;
@@ -394,24 +420,41 @@ void QQuickOpacityAnimatorJob::initialize(QQuickAnimatorController *controller)
     m_opacityNode = d->opacityNode();
     if (!m_opacityNode) {
         m_opacityNode = new QSGOpacityNode();
-        d->extra.value().opacityNode = m_opacityNode;
 
-        QSGNode *child = d->clipNode();
-        if (!child)
-            child = d->rootNode();
-        if (!child)
-            child = d->groupNode;
-
-        if (child) {
+        /* The item node subtree is like this
+         *
+         * itemNode
+         * (opacityNode)            optional
+         * (clipNode)               optional
+         * (rootNode)               optional
+         * children / paintNode
+         *
+         * If the opacity node doesn't exist, we need to insert it into
+         * the hierarchy between itemNode and clipNode or rootNode. If
+         * neither clip or root exists, we need to reparent all children
+         * from itemNode to opacityNode.
+         */
+        QSGNode *iNode = d->itemNode();
+        QSGNode *child = d->childContainerNode();
+        if (child != iNode) {
             if (child->parent())
                 child->parent()->removeChildNode(child);
             m_opacityNode->appendChildNode(child);
+            iNode->appendChildNode(m_opacityNode);
+        } else {
+            iNode->reparentChildNodesTo(m_opacityNode);
+            iNode->appendChildNode(m_opacityNode);
         }
 
-        d->itemNode()->appendChildNode(m_opacityNode);
+        d->extra.value().opacityNode = m_opacityNode;
     }
 }
 
+void QQuickOpacityAnimatorJob::nodeWasDestroyed()
+{
+    m_opacityNode = 0;
+}
+
 void QQuickOpacityAnimatorJob::writeBack()
 {
     if (m_target)
@@ -420,9 +463,9 @@ void QQuickOpacityAnimatorJob::writeBack()
 
 void QQuickOpacityAnimatorJob::updateCurrentTime(int time)
 {
-    if (!m_controller)
+    if (!m_controller || !m_opacityNode)
         return;
-    Q_ASSERT(m_controller->m_window->openglContext()->thread() == QThread::currentThread());
+    Q_ASSERT(!m_controller->m_window->openglContext() || m_controller->m_window->openglContext()->thread() == QThread::currentThread());
 
     m_value = m_from + (m_to - m_from) * m_easing.valueForProgress(time / (qreal) m_duration);
     m_opacityNode->setOpacity(m_value);
@@ -438,7 +481,7 @@ void QQuickScaleAnimatorJob::updateCurrentTime(int time)
 {
     if (!m_controller)
         return;
-    Q_ASSERT(m_controller->m_window->openglContext()->thread() == QThread::currentThread());
+    Q_ASSERT(!m_controller->m_window->openglContext() || m_controller->m_window->openglContext()->thread() == QThread::currentThread());
 
     m_value = m_from + (m_to - m_from) * m_easing.valueForProgress(time / (qreal) m_duration);
     m_helper->scale = m_value;
@@ -458,7 +501,7 @@ void QQuickRotationAnimatorJob::updateCurrentTime(int time)
 {
     if (!m_controller)
         return;
-    Q_ASSERT(m_controller->m_window->openglContext()->thread() == QThread::currentThread());
+    Q_ASSERT(!m_controller->m_window->openglContext() || m_controller->m_window->openglContext()->thread() == QThread::currentThread());
 
     float t =  m_easing.valueForProgress(time / (qreal) m_duration);
     switch (m_direction) {
@@ -504,13 +547,18 @@ void QQuickUniformAnimatorJob::setTarget(QQuickItem *target)
         m_target = target;
 }
 
+void QQuickUniformAnimatorJob::nodeWasDestroyed()
+{
+    m_node = 0;
+    m_uniformIndex = -1;
+    m_uniformType = -1;
+}
+
 void QQuickUniformAnimatorJob::afterNodeSync()
 {
     m_node = static_cast<QQuickShaderEffectNode *>(QQuickItemPrivate::get(m_target)->paintNode);
 
-    if (m_node) {
-        m_uniformIndex = -1;
-        m_uniformType = -1;
+    if (m_node && m_uniformIndex == -1 && m_uniformType == -1) {
         QQuickShaderEffectMaterial *material =
                 static_cast<QQuickShaderEffectMaterial *>(m_node->material());
         bool found = false;
@@ -533,7 +581,7 @@ void QQuickUniformAnimatorJob::updateCurrentTime(int time)
 {
     if (!m_controller)
         return;
-    Q_ASSERT(m_controller->m_window->openglContext()->thread() == QThread::currentThread());
+    Q_ASSERT(!m_controller->m_window->openglContext() || m_controller->m_window->openglContext()->thread() == QThread::currentThread());
 
     if (!m_node || m_uniformIndex == -1 || m_uniformType == -1)
         return;
