@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
@@ -10,9 +10,9 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -23,8 +23,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** $QT_END_LICENSE$
@@ -37,6 +37,7 @@
 #include "qquickanimation_p_p.h"
 #include "private/qcontinuinganimationgroupjob_p.h"
 
+#include <qmath.h>
 #include <qqmlproperty.h>
 #include <private/qqmlproperty_p.h>
 
@@ -44,7 +45,6 @@
 
 #include <QtCore/qdebug.h>
 
-#include <math.h>
 
 #define DELAY_STOP_TIMER_INTERVAL 32
 
@@ -157,7 +157,7 @@ bool QSmoothedAnimation::recalc()
         return false;
     }
 
-    finalDuration = ceil(tf * 1000.0);
+    finalDuration = qCeil(tf * 1000.0);
 
     if (maximumEasingTime == 0) {
         a = 0;
@@ -191,7 +191,7 @@ bool QSmoothedAnimation::recalc()
         qreal c2 = 0.5 * vi * tf - s;
         qreal c3 = -0.25 * vi * vi;
 
-        qreal a1 = (-c2 + sqrt(c2 * c2 - 4 * c1 * c3)) / (2. * c1);
+        qreal a1 = (-c2 + qSqrt(c2 * c2 - 4 * c1 * c3)) / (2. * c1);
 
         qreal tp1 = 0.5 * tf - 0.5 * vi / a1;
         qreal vp1 = a1 * tp1 + vi;
@@ -307,7 +307,7 @@ void QSmoothedAnimation::init()
 
 void QSmoothedAnimation::debugAnimation(QDebug d) const
 {
-    d << "SmoothedAnimationJob(" << hex << (void *) this << dec << ")" << "duration:" << userDuration
+    d << "SmoothedAnimationJob(" << hex << (const void *) this << dec << ")" << "duration:" << userDuration
       << "velocity:" << velocity << "target:" << target.object() << "property:" << target.name()
       << "to:" << to << "current velocity:" << trackVelocity;
 }
@@ -378,11 +378,11 @@ QQuickSmoothedAnimationPrivate::QQuickSmoothedAnimationPrivate()
 
 QQuickSmoothedAnimationPrivate::~QQuickSmoothedAnimationPrivate()
 {
+    typedef QHash<QQmlProperty, QSmoothedAnimation* >::iterator ActiveAnimationsHashIt;
+
     delete anim;
-    QHash<QQmlProperty, QSmoothedAnimation* >::iterator it;
-    for (it = activeAnimations.begin(); it != activeAnimations.end(); ++it) {
+    for (ActiveAnimationsHashIt it = activeAnimations.begin(), end = activeAnimations.end(); it != end; ++it)
         it.value()->clearTemplate();
-    }
 }
 
 void QQuickSmoothedAnimationPrivate::updateRunningAnimations()
