@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
@@ -10,9 +10,9 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -23,8 +23,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** $QT_END_LICENSE$
@@ -37,11 +37,12 @@
 #include <QDebug>
 #include <QScopedPointer>
 #include <QNetworkCookieJar>
+#include <QThread>
+#include <QMutex>
+#include <QWaitCondition>
 #include "testhttpserver.h"
 #include "../../shared/util.h"
 
-#define SERVER_PORT 14445
-
 class tst_qqmlxmlhttprequest : public QQmlDataTest
 {
     Q_OBJECT
@@ -73,6 +74,8 @@ private slots:
     void send_ignoreData();
     void send_withdata();
     void send_withdata_data();
+    void send_options();
+    void send_options_data();
     void abort();
     void abort_unsent();
     void abort_opened();
@@ -84,6 +87,7 @@ private slots:
     void getAllResponseHeaders_unsent();
     void getAllResponseHeaders_sent();
     void getAllResponseHeaders_args();
+    void getBinaryData();
     void status();
     void status_data();
     void statusText();
@@ -234,10 +238,11 @@ void tst_qqmlxmlhttprequest::open()
 
     TestHTTPServer server;
     if (remote) {
-        QVERIFY2(server.listen(SERVER_PORT), qPrintable(server.errorString()));
+        QVERIFY2(server.listen(), qPrintable(server.errorString()));
         QVERIFY(server.wait(testFileUrl("open_network.expect"),
                             testFileUrl("open_network.reply"),
                             testFileUrl("testdocument.html")));
+        url = server.urlString(url);
     }
 
     QQmlComponent component(&engine, qmlFile);
@@ -264,10 +269,10 @@ void tst_qqmlxmlhttprequest::open_data()
 
     QTest::newRow("Relative url)") << testFileUrl("open.qml") << "testdocument.html" << false;
     QTest::newRow("Absolute url)") << testFileUrl("open.qml") << testFileUrl("testdocument.html").toString() << false;
-    QTest::newRow("Absolute network url)") << testFileUrl("open.qml") << "http://127.0.0.1:14445/testdocument.html" << true;
+    QTest::newRow("Absolute network url)") << testFileUrl("open.qml") << "/testdocument.html" << true;
 
     // ### Check that the username/password were sent to the server
-    QTest::newRow("User/pass") << testFileUrl("open_user.qml") << "http://127.0.0.1:14445/testdocument.html" << true;
+    QTest::newRow("User/pass") << testFileUrl("open_user.qml") << "/testdocument.html" << true;
 }
 
 // Test that calling XMLHttpRequest.open() with an invalid method raises an exception
@@ -280,14 +285,55 @@ void tst_qqmlxmlhttprequest::open_invalid_method()
     QCOMPARE(object->property("exceptionThrown").toBool(), true);
 }
 
-// Test that calling XMLHttpRequest.open() with sync raises an exception
+class TestThreadedHTTPServer : public QObject
+{
+    Q_OBJECT
+public:
+    TestThreadedHTTPServer(const QUrl &expectUrl, const QUrl &replyUrl, const QUrl &bodyUrl)
+        : m_server(Q_NULLPTR) {
+        QMutexLocker locker(&m_lock);
+        moveToThread(&m_thread);
+        m_thread.start();
+        QMetaObject::invokeMethod(this, "start", Qt::QueuedConnection, Q_ARG(QUrl, expectUrl), Q_ARG(QUrl, replyUrl), Q_ARG(QUrl, bodyUrl));
+        m_startupCondition.wait(&m_lock);
+    }
+    ~TestThreadedHTTPServer() {
+        m_server->deleteLater();
+        m_thread.exit();
+        m_thread.wait();
+    }
+
+    QUrl serverBaseUrl;
+
+private slots:
+    void start(const QUrl &expectUrl, const QUrl &replyUrl, const QUrl &bodyUrl) {
+        QMutexLocker locker(&m_lock);
+        m_server = new TestHTTPServer;
+        QVERIFY2(m_server->listen(), qPrintable(m_server->errorString()));
+        serverBaseUrl = m_server->baseUrl();
+        QVERIFY(m_server->wait(expectUrl, replyUrl, bodyUrl));
+        m_startupCondition.wakeAll();
+    }
+
+private:
+    TestHTTPServer *m_server;
+    QThread m_thread;
+    QMutex m_lock;
+    QWaitCondition m_startupCondition;
+};
+
+// Test that calling XMLHttpRequest.open() with sync
 void tst_qqmlxmlhttprequest::open_sync()
 {
-    QQmlComponent component(&engine, testFileUrl("open_sync.qml"));
-    QScopedPointer<QObject> object(component.create());
-    QVERIFY(!object.isNull());
+    TestThreadedHTTPServer server(testFileUrl("open_network.expect"), testFileUrl("open_network.reply"), testFileUrl("testdocument.html"));
 
-    QCOMPARE(object->property("exceptionThrown").toBool(), true);
+    QQmlComponent component(&engine, testFileUrl("open_sync.qml"));
+    QScopedPointer<QObject> object(component.beginCreate(engine.rootContext()));
+    QVERIFY(!object.isNull());
+    object->setProperty("url", server.serverBaseUrl.resolved(QStringLiteral("/testdocument.html")).toString());
+    component.completeCreate();
+
+    QCOMPARE(object->property("responseText").toString(), QStringLiteral("QML Rocks!\n"));
 }
 
 // Calling with incorrect arg count raises an exception
@@ -314,7 +360,7 @@ void tst_qqmlxmlhttprequest::open_arg_count()
 void tst_qqmlxmlhttprequest::setRequestHeader()
 {
     TestHTTPServer server;
-    QVERIFY2(server.listen(SERVER_PORT), qPrintable(server.errorString()));
+    QVERIFY2(server.listen(), qPrintable(server.errorString()));
     QVERIFY(server.wait(testFileUrl("setRequestHeader.expect"),
                         testFileUrl("setRequestHeader.reply"),
                         testFileUrl("testdocument.html")));
@@ -322,7 +368,7 @@ void tst_qqmlxmlhttprequest::setRequestHeader()
     QQmlComponent component(&engine, testFileUrl("setRequestHeader.qml"));
     QScopedPointer<QObject> object(component.beginCreate(engine.rootContext()));
     QVERIFY(!object.isNull());
-    object->setProperty("url", "http://127.0.0.1:14445/testdocument.html");
+    object->setProperty("url", server.urlString("/testdocument.html"));
     component.completeCreate();
 
     QTRY_VERIFY(object->property("dataOK").toBool() == true);
@@ -332,7 +378,7 @@ void tst_qqmlxmlhttprequest::setRequestHeader()
 void tst_qqmlxmlhttprequest::setRequestHeader_caseInsensitive()
 {
     TestHTTPServer server;
-    QVERIFY2(server.listen(SERVER_PORT), qPrintable(server.errorString()));
+    QVERIFY2(server.listen(), qPrintable(server.errorString()));
     QVERIFY(server.wait(testFileUrl("setRequestHeader.expect"),
                         testFileUrl("setRequestHeader.reply"),
                         testFileUrl("testdocument.html")));
@@ -340,7 +386,7 @@ void tst_qqmlxmlhttprequest::setRequestHeader_caseInsensitive()
     QQmlComponent component(&engine, testFileUrl("setRequestHeader_caseInsensitive.qml"));
     QScopedPointer<QObject> object(component.beginCreate(engine.rootContext()));
     QVERIFY(!object.isNull());
-    object->setProperty("url", "http://127.0.0.1:14445/testdocument.html");
+    object->setProperty("url", server.urlString("/testdocument.html"));
     component.completeCreate();
 
     QTRY_VERIFY(object->property("dataOK").toBool() == true);
@@ -389,7 +435,7 @@ void tst_qqmlxmlhttprequest::setRequestHeader_illegalName()
     QFETCH(QString, name);
 
     TestHTTPServer server;
-    QVERIFY2(server.listen(SERVER_PORT), qPrintable(server.errorString()));
+    QVERIFY2(server.listen(), qPrintable(server.errorString()));
     QVERIFY(server.wait(testFileUrl("open_network.expect"),
                         testFileUrl("open_network.reply"),
                         testFileUrl("testdocument.html")));
@@ -397,7 +443,7 @@ void tst_qqmlxmlhttprequest::setRequestHeader_illegalName()
     QQmlComponent component(&engine, testFileUrl("setRequestHeader_illegalName.qml"));
     QScopedPointer<QObject> object(component.beginCreate(engine.rootContext()));
     QVERIFY(!object.isNull());
-    object->setProperty("url", "http://127.0.0.1:14445/testdocument.html");
+    object->setProperty("url", server.urlString("/testdocument.html"));
     object->setProperty("header", name);
     component.completeCreate();
 
@@ -415,7 +461,7 @@ void tst_qqmlxmlhttprequest::setRequestHeader_illegalName()
 void tst_qqmlxmlhttprequest::setRequestHeader_sent()
 {
     TestHTTPServer server;
-    QVERIFY2(server.listen(SERVER_PORT), qPrintable(server.errorString()));
+    QVERIFY2(server.listen(), qPrintable(server.errorString()));
     QVERIFY(server.wait(testFileUrl("open_network.expect"),
                         testFileUrl("open_network.reply"),
                         testFileUrl("testdocument.html")));
@@ -423,7 +469,7 @@ void tst_qqmlxmlhttprequest::setRequestHeader_sent()
     QQmlComponent component(&engine, testFileUrl("setRequestHeader_sent.qml"));
     QScopedPointer<QObject> object(component.beginCreate(engine.rootContext()));
     QVERIFY(!object.isNull());
-    object->setProperty("url", "http://127.0.0.1:14445/testdocument.html");
+    object->setProperty("url", server.urlString("/testdocument.html"));
     component.completeCreate();
 
     QCOMPARE(object->property("test").toBool(), true);
@@ -467,7 +513,7 @@ void tst_qqmlxmlhttprequest::send_ignoreData()
 {
     {
         TestHTTPServer server;
-        QVERIFY2(server.listen(SERVER_PORT), qPrintable(server.errorString()));
+        QVERIFY2(server.listen(), qPrintable(server.errorString()));
         QVERIFY(server.wait(testFileUrl("send_ignoreData_GET.expect"),
                             testFileUrl("send_ignoreData.reply"),
                             testFileUrl("testdocument.html")));
@@ -476,7 +522,7 @@ void tst_qqmlxmlhttprequest::send_ignoreData()
         QScopedPointer<QObject> object(component.beginCreate(engine.rootContext()));
         QVERIFY(!object.isNull());
         object->setProperty("reqType", "GET");
-        object->setProperty("url", "http://127.0.0.1:14445/testdocument.html");
+        object->setProperty("url", server.urlString("/testdocument.html"));
         component.completeCreate();
 
         QTRY_VERIFY(object->property("dataOK").toBool() == true);
@@ -484,7 +530,7 @@ void tst_qqmlxmlhttprequest::send_ignoreData()
 
     {
         TestHTTPServer server;
-        QVERIFY2(server.listen(SERVER_PORT), qPrintable(server.errorString()));
+        QVERIFY2(server.listen(), qPrintable(server.errorString()));
         QVERIFY(server.wait(testFileUrl("send_ignoreData_HEAD.expect"),
                             testFileUrl("send_ignoreData.reply"),
                             QUrl()));
@@ -493,7 +539,7 @@ void tst_qqmlxmlhttprequest::send_ignoreData()
         QScopedPointer<QObject> object(component.beginCreate(engine.rootContext()));
         QVERIFY(!object.isNull());
         object->setProperty("reqType", "HEAD");
-        object->setProperty("url", "http://127.0.0.1:14445/testdocument.html");
+        object->setProperty("url", server.urlString("/testdocument.html"));
         component.completeCreate();
 
         QTRY_VERIFY(object->property("dataOK").toBool() == true);
@@ -501,7 +547,7 @@ void tst_qqmlxmlhttprequest::send_ignoreData()
 
     {
         TestHTTPServer server;
-        QVERIFY2(server.listen(SERVER_PORT), qPrintable(server.errorString()));
+        QVERIFY2(server.listen(), qPrintable(server.errorString()));
         QVERIFY(server.wait(testFileUrl("send_ignoreData_DELETE.expect"),
                             testFileUrl("send_ignoreData.reply"),
                             QUrl()));
@@ -510,7 +556,7 @@ void tst_qqmlxmlhttprequest::send_ignoreData()
         QScopedPointer<QObject> object(component.beginCreate(engine.rootContext()));
         QVERIFY(!object.isNull());
         object->setProperty("reqType", "DELETE");
-        object->setProperty("url", "http://127.0.0.1:14445/testdocument.html");
+        object->setProperty("url", server.urlString("/testdocument.html"));
         component.completeCreate();
 
         QTRY_VERIFY(object->property("dataOK").toBool() == true);
@@ -524,7 +570,7 @@ void tst_qqmlxmlhttprequest::send_withdata()
     QFETCH(QString, file_qml);
 
     TestHTTPServer server;
-    QVERIFY2(server.listen(SERVER_PORT), qPrintable(server.errorString()));
+    QVERIFY2(server.listen(), qPrintable(server.errorString()));
     QVERIFY(server.wait(testFileUrl(file_expected),
                         testFileUrl("send_data.reply"),
                         testFileUrl("testdocument.html")));
@@ -532,7 +578,7 @@ void tst_qqmlxmlhttprequest::send_withdata()
     QQmlComponent component(&engine, testFileUrl(file_qml));
     QScopedPointer<QObject> object(component.beginCreate(engine.rootContext()));
     QVERIFY(!object.isNull());
-    object->setProperty("url", "http://127.0.0.1:14445/testdocument.html");
+    object->setProperty("url", server.urlString("/testdocument.html"));
     component.completeCreate();
 
     QTRY_VERIFY(object->property("dataOK").toBool() == true);
@@ -552,6 +598,47 @@ void tst_qqmlxmlhttprequest::send_withdata_data()
     QTest::newRow("Correct content-type - no charset") << "send_data.1.expect" << "send_data.7.qml";
 }
 
+void tst_qqmlxmlhttprequest::send_options()
+{
+    QFETCH(QString, url_suffix);
+    QFETCH(QString, file_expected);
+    QFETCH(QString, file_qml);
+    QFETCH(QString, file_reply);
+
+    TestHTTPServer server;
+    QVERIFY2(server.listen(), qPrintable(server.errorString()));
+    QVERIFY(server.wait(testFileUrl(file_expected),
+                        testFileUrl(file_reply),
+                        testFileUrl("testdocument.html")));
+
+    QQmlComponent component(&engine, testFileUrl(file_qml));
+    QScopedPointer<QObject> object(component.beginCreate(engine.rootContext()));
+    QVERIFY(!object.isNull());
+    QString url = server.baseUrl().toString();
+    if (url_suffix != "/")
+        url.append("/");
+    if (!url_suffix.isEmpty())
+        url.append(url_suffix);
+    object->setProperty("url", url);
+    component.completeCreate();
+
+    QTRY_VERIFY(object->property("dataOK").toBool());
+    QTRY_VERIFY(object->property("headerOK").toBool());
+}
+
+void tst_qqmlxmlhttprequest::send_options_data()
+{
+    QTest::addColumn<QString>("url_suffix");
+    QTest::addColumn<QString>("file_expected");
+    QTest::addColumn<QString>("file_qml");
+    QTest::addColumn<QString>("file_reply");
+
+    QTest::newRow("OPTIONS (no data, no resource, no path)") << "" << "send_data.8.expect" << "send_data.8.qml" << "send_data.2.reply";
+    QTest::newRow("OPTIONS (no data, no resource, path- \"/\")") << "/" << "send_data.8.expect" << "send_data.8.qml" << "send_data.2.reply";
+    QTest::newRow("OPTIONS (no data, with resource)") << "testdocument.html" << "send_data.9.expect" << "send_data.8.qml" << "send_data.2.reply";
+    QTest::newRow("OPTIONS (with data)") << "testdocument.html" << "send_data.10.expect" << "send_data.9.qml" << "send_data.2.reply";
+}
+
 // Test abort() has no effect in unsent state
 void tst_qqmlxmlhttprequest::abort_unsent()
 {
@@ -594,7 +681,7 @@ void tst_qqmlxmlhttprequest::abort_opened()
 void tst_qqmlxmlhttprequest::abort()
 {
     TestHTTPServer server;
-    QVERIFY2(server.listen(SERVER_PORT), qPrintable(server.errorString()));
+    QVERIFY2(server.listen(), qPrintable(server.errorString()));
     QVERIFY(server.wait(testFileUrl("abort.expect"),
                         testFileUrl("abort.reply"),
                         testFileUrl("testdocument.html")));
@@ -602,8 +689,11 @@ void tst_qqmlxmlhttprequest::abort()
     QQmlComponent component(&engine, testFileUrl("abort.qml"));
     QScopedPointer<QObject> object(component.beginCreate(engine.rootContext()));
     QVERIFY(!object.isNull());
-    object->setProperty("urlDummy", "http://127.0.0.1:14449/testdocument.html");
-    object->setProperty("url", "http://127.0.0.1:14445/testdocument.html");
+    const QUrl url = server.url("/testdocument.html");
+    QUrl dummyUrl = url;
+    dummyUrl.setPort(dummyUrl.port() - 1);
+    object->setProperty("urlDummy", dummyUrl.toString());
+    object->setProperty("url", url.toString());
     component.completeCreate();
 
     QCOMPARE(object->property("seenDone").toBool(), true);
@@ -618,7 +708,7 @@ void tst_qqmlxmlhttprequest::getResponseHeader()
     QQmlEngine engine; // Avoid cookie contamination
 
     TestHTTPServer server;
-    QVERIFY2(server.listen(SERVER_PORT), qPrintable(server.errorString()));
+    QVERIFY2(server.listen(), qPrintable(server.errorString()));
     QVERIFY(server.wait(testFileUrl("getResponseHeader.expect"),
                         testFileUrl("getResponseHeader.reply"),
                         testFileUrl("testdocument.html")));
@@ -627,7 +717,7 @@ void tst_qqmlxmlhttprequest::getResponseHeader()
     QQmlComponent component(&engine, testFileUrl("getResponseHeader.qml"));
     QScopedPointer<QObject> object(component.beginCreate(engine.rootContext()));
     QVERIFY(!object.isNull());
-    object->setProperty("url", "http://127.0.0.1:14445/testdocument.html");
+    object->setProperty("url", server.urlString("/testdocument.html"));
     component.completeCreate();
 
     QCOMPARE(object->property("unsentException").toBool(), true);
@@ -685,7 +775,7 @@ void tst_qqmlxmlhttprequest::getAllResponseHeaders()
     QQmlEngine engine; // Avoid cookie contamination
 
     TestHTTPServer server;
-    QVERIFY2(server.listen(SERVER_PORT), qPrintable(server.errorString()));
+    QVERIFY2(server.listen(), qPrintable(server.errorString()));
     QVERIFY(server.wait(testFileUrl("getResponseHeader.expect"),
                         testFileUrl("getResponseHeader.reply"),
                         testFileUrl("testdocument.html")));
@@ -693,7 +783,7 @@ void tst_qqmlxmlhttprequest::getAllResponseHeaders()
     QQmlComponent component(&engine, testFileUrl("getAllResponseHeaders.qml"));
     QScopedPointer<QObject> object(component.beginCreate(engine.rootContext()));
     QVERIFY(!object.isNull());
-    object->setProperty("url", "http://127.0.0.1:14445/testdocument.html");
+    object->setProperty("url", server.urlString("/testdocument.html"));
     component.completeCreate();
 
     QCOMPARE(object->property("unsentException").toBool(), true);
@@ -740,13 +830,31 @@ void tst_qqmlxmlhttprequest::getAllResponseHeaders_args()
     QTRY_VERIFY(object->property("exceptionThrown").toBool() == true);
 }
 
+void tst_qqmlxmlhttprequest::getBinaryData()
+{
+    TestHTTPServer server;
+    QVERIFY2(server.listen(), qPrintable(server.errorString()));
+    QVERIFY(server.wait(testFileUrl("receive_binary_data.expect"),
+                        testFileUrl("receive_binary_data.reply"),
+                        testFileUrl("qml_logo.png")));
+
+    QQmlComponent component(&engine, testFileUrl("receiveBinaryData.qml"));
+    QScopedPointer<QObject> object(component.beginCreate(engine.rootContext()));
+    QVERIFY(!object.isNull());
+    object->setProperty("url", server.urlString("/gml_logo.png"));
+    component.completeCreate();
+
+    QFileInfo fileInfo("data/qml_logo.png");
+    QTRY_VERIFY(object->property("readSize").toInt() == fileInfo.size());
+}
+
 void tst_qqmlxmlhttprequest::status()
 {
     QFETCH(QUrl, replyUrl);
     QFETCH(int, status);
 
     TestHTTPServer server;
-    QVERIFY2(server.listen(SERVER_PORT), qPrintable(server.errorString()));
+    QVERIFY2(server.listen(), qPrintable(server.errorString()));
     QVERIFY(server.wait(testFileUrl("status.expect"),
                         replyUrl,
                         testFileUrl("testdocument.html")));
@@ -754,7 +862,7 @@ void tst_qqmlxmlhttprequest::status()
     QQmlComponent component(&engine, testFileUrl("status.qml"));
     QScopedPointer<QObject> object(component.beginCreate(engine.rootContext()));
     QVERIFY(!object.isNull());
-    object->setProperty("url", "http://127.0.0.1:14445/testdocument.html");
+    object->setProperty("url", server.urlString("/testdocument.html"));
     object->setProperty("expectedStatus", status);
     component.completeCreate();
 
@@ -785,7 +893,7 @@ void tst_qqmlxmlhttprequest::statusText()
     QFETCH(QString, statusText);
 
     TestHTTPServer server;
-    QVERIFY2(server.listen(SERVER_PORT), qPrintable(server.errorString()));
+    QVERIFY2(server.listen(), qPrintable(server.errorString()));
     QVERIFY(server.wait(testFileUrl("status.expect"),
                         replyUrl,
                         testFileUrl("testdocument.html")));
@@ -793,7 +901,7 @@ void tst_qqmlxmlhttprequest::statusText()
     QQmlComponent component(&engine, testFileUrl("statusText.qml"));
     QScopedPointer<QObject> object(component.beginCreate(engine.rootContext()));
     QVERIFY(!object.isNull());
-    object->setProperty("url", "http://127.0.0.1:14445/testdocument.html");
+    object->setProperty("url", server.urlString("/testdocument.html"));
     object->setProperty("expectedStatus", statusText);
     component.completeCreate();
 
@@ -825,7 +933,7 @@ void tst_qqmlxmlhttprequest::responseText()
     QFETCH(QString, responseText);
 
     TestHTTPServer server;
-    QVERIFY2(server.listen(SERVER_PORT), qPrintable(server.errorString()));
+    QVERIFY2(server.listen(), qPrintable(server.errorString()));
     QVERIFY(server.wait(testFileUrl("status.expect"),
                         replyUrl,
                         bodyUrl));
@@ -833,7 +941,7 @@ void tst_qqmlxmlhttprequest::responseText()
     QQmlComponent component(&engine, testFileUrl("responseText.qml"));
     QScopedPointer<QObject> object(component.beginCreate(engine.rootContext()));
     QVERIFY(!object.isNull());
-    object->setProperty("url", "http://127.0.0.1:14445/testdocument.html");
+    object->setProperty("url", server.urlString("/testdocument.html"));
     object->setProperty("expectedText", responseText);
     component.completeCreate();
 
@@ -926,14 +1034,14 @@ void tst_qqmlxmlhttprequest::redirects()
 {
     {
         TestHTTPServer server;
-        QVERIFY2(server.listen(SERVER_PORT), qPrintable(server.errorString()));
-        server.addRedirect("redirect.html", "http://127.0.0.1:14445/redirecttarget.html");
+        QVERIFY2(server.listen(), qPrintable(server.errorString()));
+        server.addRedirect("redirect.html", server.urlString("/redirecttarget.html"));
         server.serveDirectory(dataDirectory());
 
         QQmlComponent component(&engine, testFileUrl("redirects.qml"));
         QScopedPointer<QObject> object(component.beginCreate(engine.rootContext()));
         QVERIFY(!object.isNull());
-        object->setProperty("url", "http://127.0.0.1:14445/redirect.html");
+        object->setProperty("url", server.urlString("/redirect.html"));
         object->setProperty("expectedText", "");
         component.completeCreate();
 
@@ -943,14 +1051,14 @@ void tst_qqmlxmlhttprequest::redirects()
 
     {
         TestHTTPServer server;
-        QVERIFY2(server.listen(SERVER_PORT), qPrintable(server.errorString()));
-        server.addRedirect("redirect.html", "http://127.0.0.1:14445/redirectmissing.html");
+        QVERIFY2(server.listen(), qPrintable(server.errorString()));
+        server.addRedirect("redirect.html", server.urlString("/redirectmissing.html"));
         server.serveDirectory(dataDirectory());
 
         QQmlComponent component(&engine, testFileUrl("redirectError.qml"));
         QScopedPointer<QObject> object(component.beginCreate(engine.rootContext()));
         QVERIFY(!object.isNull());
-        object->setProperty("url", "http://127.0.0.1:14445/redirect.html");
+        object->setProperty("url", server.urlString("/redirect.html"));
         object->setProperty("expectedText", "");
         component.completeCreate();
 
@@ -960,14 +1068,14 @@ void tst_qqmlxmlhttprequest::redirects()
 
     {
         TestHTTPServer server;
-        QVERIFY2(server.listen(SERVER_PORT), qPrintable(server.errorString()));
-        server.addRedirect("redirect.html", "http://127.0.0.1:14445/redirect.html");
+        QVERIFY2(server.listen(), qPrintable(server.errorString()));
+        server.addRedirect("redirect.html", server.urlString("/redirect.html"));
         server.serveDirectory(dataDirectory());
 
         QQmlComponent component(&engine, testFileUrl("redirectRecur.qml"));
         QScopedPointer<QObject> object(component.beginCreate(engine.rootContext()));
         QVERIFY(!object.isNull());
-        object->setProperty("url", "http://127.0.0.1:14445/redirect.html");
+        object->setProperty("url", server.urlString("/redirect.html"));
         object->setProperty("expectedText", "");
         component.completeCreate();
 
@@ -1062,12 +1170,14 @@ void tst_qqmlxmlhttprequest::stateChangeCallingContext()
     // without a valid calling context.
 
     TestHTTPServer server;
-    QVERIFY2(server.listen(SERVER_PORT), qPrintable(server.errorString()));
+    QVERIFY2(server.listen(), qPrintable(server.errorString()));
     server.serveDirectory(dataDirectory(), TestHTTPServer::Delay);
 
     QQmlComponent component(&engine, testFileUrl("stateChangeCallingContext.qml"));
-    QScopedPointer<QObject> object(component.create());
+    QScopedPointer<QObject> object(component.beginCreate(engine.rootContext()));
     QVERIFY(!object.isNull());
+    object->setProperty("serverBaseUrl", server.baseUrl().toString());
+    component.completeCreate();
     server.sendDelayedItem();
     QTRY_VERIFY(object->property("success").toBool() == true);
 }
