@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
@@ -10,9 +10,9 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -23,8 +23,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** $QT_END_LICENSE$
@@ -37,6 +37,7 @@
 #include <QtQuick/qquickitem.h>
 #include <QtQuick/qquickview.h>
 #include <QtGui/qinputmethod.h>
+#include <QtGui/qstylehints.h>
 #include <qpa/qwindowsysteminterface.h>
 #include <qpa/qplatformintegration.h>
 #include <private/qguiapplication_p.h>
@@ -52,6 +53,7 @@ private slots:
     void state();
     void layoutDirection();
     void inputMethod();
+    void styleHints();
     void cleanup();
 
 private:
@@ -132,31 +134,30 @@ void tst_qquickapplication::state()
     QQuickWindow window;
     item->setParentItem(window.contentItem());
 
-    // initial state should be ApplicationInactive
-    QCOMPARE(Qt::ApplicationState(item->property("state").toInt()), Qt::ApplicationInactive);
-    QCOMPARE(Qt::ApplicationState(item->property("state2").toInt()), Qt::ApplicationInactive);
-
     // If the platform plugin has the ApplicationState capability, state changes originate from it
     // as a result of a system event. We therefore have to simulate these events here.
     if (QGuiApplicationPrivate::platformIntegration()->hasCapability(QPlatformIntegration::ApplicationState)) {
 
+        // Flush pending events, in case the platform have already queued real application state events
+        QWindowSystemInterface::flushWindowSystemEvents();
+
         QWindowSystemInterface::handleApplicationStateChanged(Qt::ApplicationActive);
-        QTest::waitForEvents();
+        QWindowSystemInterface::flushWindowSystemEvents();
         QCOMPARE(Qt::ApplicationState(item->property("state").toInt()), Qt::ApplicationActive);
         QCOMPARE(Qt::ApplicationState(item->property("state2").toInt()), Qt::ApplicationActive);
 
         QWindowSystemInterface::handleApplicationStateChanged(Qt::ApplicationInactive);
-        QTest::waitForEvents();
+        QWindowSystemInterface::flushWindowSystemEvents();
         QCOMPARE(Qt::ApplicationState(item->property("state").toInt()), Qt::ApplicationInactive);
         QCOMPARE(Qt::ApplicationState(item->property("state2").toInt()), Qt::ApplicationInactive);
 
         QWindowSystemInterface::handleApplicationStateChanged(Qt::ApplicationSuspended);
-        QTest::waitForEvents();
+        QWindowSystemInterface::flushWindowSystemEvents();
         QCOMPARE(Qt::ApplicationState(item->property("state").toInt()), Qt::ApplicationSuspended);
         QCOMPARE(Qt::ApplicationState(item->property("state2").toInt()), Qt::ApplicationSuspended);
 
         QWindowSystemInterface::handleApplicationStateChanged(Qt::ApplicationHidden);
-        QTest::waitForEvents();
+        QWindowSystemInterface::flushWindowSystemEvents();
         QCOMPARE(Qt::ApplicationState(item->property("state").toInt()), Qt::ApplicationHidden);
         QCOMPARE(Qt::ApplicationState(item->property("state2").toInt()), Qt::ApplicationHidden);
 
@@ -214,6 +215,18 @@ void tst_qquickapplication::inputMethod()
     QCOMPARE(qvariant_cast<QObject*>(item->property("inputMethod")), qApp->inputMethod());
 }
 
+void tst_qquickapplication::styleHints()
+{
+    // technically not in QQuickApplication, but testing anyway here
+    QQmlComponent component(&engine);
+    component.setData("import QtQuick 2.0; Item { property variant styleHints: Qt.styleHints }", QUrl::fromLocalFile(""));
+    QQuickItem *item = qobject_cast<QQuickItem *>(component.create());
+    QVERIFY(item);
+    QQuickView view;
+    item->setParentItem(view.rootObject());
+
+    QCOMPARE(qvariant_cast<QObject*>(item->property("styleHints")), qApp->styleHints());
+}
 
 QTEST_MAIN(tst_qquickapplication)
 
